/// \file EventBusyBox.ino
///
/// \copyright Copyright (c) 2017, Garth Zeglin. All rights reserved. Licensed
/// under the terms of the BSD 3-clause license as included in LICENSE.
///
/// \brief Omnibus Arduino sketch demonstrating event-loop programming templates (work-in-progress).
///
/// This multi-file Arduino sketch demonstrates a variety of working examples
/// for performing non-blocking event-driven I/O.  It is intended as a starting
/// point for projects which combine responsive sensing and actuation with
/// behavioral logic.
///
/// N.B. This is a work in progress and only partly tested.
///
/// This top-level file just contains the minimal common code, with most of the
/// function calls commented out.  To use this template, you will need to pick
/// the specific modules you need for your project, update the hardware pin
/// assignments to match, and uncomment the related function calls from setup()
/// and poll().  You may then remove any unnecessary code, i.e. delete unused
/// files and unnecessary comments.
///
/// This opt-in approach is intended to discourage the usual problem of examples
/// in which a student isn't sure which parts are needed and so leaves lots of
/// unnecessary sample code in place even if it interferes with the intended
/// outcome.
///
/// The different functional modules are stored in different .ino files which
/// will appear in the Arduino IDE as separate tabs.  This keeps related
/// definitions together but without introducing C++ class and objects.  Please
/// note that the Arduino system combines multiple .ino files into one unit
/// before compiling, so the code could be combined into a single .ino file if
/// desired without much trouble.

// ================================================================================

/// Standard Arduino entry point to configure the hardware once after booting
/// up.  This runs once after pressing reset or powering up the board.

void setup(void)
{
  // Initialize the Serial port for the user debugging console.  Note the fast baud rate for low latency.
  Serial.begin(115200);

  // Please uncomment the function calls for the specific modules you wish to use:
  setup_LED_blinker();
  // setup_melody_player();
  // setup_servo_sweeper();
  // setup_servo_animation();
  // setup_sonar();
  // setup_speed_stepper();

}
// ================================================================================

/// Top-level polling entry point for servicing all background I/O and periodic
/// processing.  This function should be called repeatedly as fast as possible
/// from within loop() to poll program events.  This loop should never be
/// allowed to stall or block so that all tasks can be constantly serviced.  In
/// particular, the top-level loop should use polling_delay() instead of delay()
/// which would otherwise stall all activity.

void poll(void)
{
  /// The timestamp in microseconds from the previous polling cycle, used to
  /// compute the interval between updates.
  static unsigned long last_update_clock = 0;
   
  /// The current clock time in microseconds.
  unsigned long now = micros();

  // Compute the time elapsed in microseconds since the last poll.  This will
  // correctly handle wrapround of the 32-bit long time value given the
  // properties of twos-complement arithmetic.
  unsigned long interval = now - last_update_clock;
  last_update_clock = now;

  // Begin the polling cycle.  This passes the elapsed time value to each module
  // so it can independently evaluate when to poll inputs or update outputs
  // using local timer variables.

  // Please uncomment the function calls for the specific modules you wish to use:
  poll_console_input(interval);
  poll_LED_blinker(interval);
  // poll_melody_player(interval);
  // poll_servo_sweeper(interval);
  // poll_servo_animation(interval);
  // poll_sonar(interval);
  // poll_speed_stepper(interval);
}
// ================================================================================

/// Wait for the given number of milliseconds while servicing background
/// activity.  This is a replacement for delay() for use within loop().  Note
/// that should not be called from polling functions, only from loop().
///
///   \param milliseconds number of milliseconds to poll before returning

void polling_delay(long milliseconds)
{
  // Record initial clock time in milliseconds.
  unsigned long then = millis();

  // Keep polling as long as the time remaining is positive.
  while (milliseconds > 0) {

    // Update all I/O functions.
    poll();

    // Calculate the elapsed time and subtract it from the delay counter.
    unsigned long now = millis();    
    milliseconds -= (now - then);
    then = now;
  }
}
   
// ================================================================================

/// Standard Arduino program entry, called repeatedly from the standard library.
/// This will normally hold your top-level behavior script or operational loop.
/// However, if this script needs to pause or wait for an event, it should make
/// sure that the poll() function continues to be called as fast as possible to
/// service all I/O activity.  In particular, polling_delay() should be used
/// instead of delay().

void loop(void)
{
  // The following sample code will be replaced by your primary logic script.
  // Whenever the logic needs to wait on an event or delay, it should be sure to
  // call poll() or polling_delay() to keep checking for events and updating the
  // I/O state.

  Serial.println("start-of-loop");

  polling_delay(1000);   // Wait for a second.
  Serial.println("starting-logic");

  // Wait a few times for the LED blinker state to change.
  for (int i=0; i < 10; i++) {
    while(!is_LED_blinker_on()) poll(); // wait until the LED blinker reports ON
    Serial.println("led-now-on");

    while(is_LED_blinker_on()) poll(); // wait until the LED blinker reports OFF
    Serial.println("led-now-off");
  }
  
  Serial.println("end-of-loop");
}
// ================================================================================
