/// \file sonar.ino
/// \brief Sonar sensor demo module for EventBusyBox illustrating event-loop
/// programming.

// \copyright Copyright (c) 2017, Garth Zeglin. All rights reserved. Licensed
// under the terms of the BSD 3-clause license as included in LICENSE.

// ================================================================================
// Global state variables for the module.

/// The sonar trigger pin output assignment.
const int TRIG_PIN = 4;

/// The sonar echo pin input assignment.
const int ECHO_PIN = 5;

/// The rated distance limit of the sensor, in cm.
const int MAX_DISTANCE = 450;

/// A typical speed of sound, specified in cm/sec.
const long SOUND_SPEED = 34000;

/// Determine the maximum time to wait for an echo. The maximum rated distance is
/// 4.5 meters; if no echo is received within the duration representing this
/// round-trip distance, stop measuring.  The timeout is specified in
/// microseconds.
const long TIMEOUT = (2 * MAX_DISTANCE * 1000000)/SOUND_SPEED;

/// Time remaining in microseconds before the next sonar cycle.
long sonar_timer = 0;

/// The number of microseconds betwen sonar cycles.
const long sonar_interval = 200000;

/// The most recent sonar echo distance in centimeters, or -1 if no echo was detected.
int sonar_distance = 0;

// ================================================================================
/// Configuration function to call from setup().
void setup_sonar(void)
{
  // Initialize the trigger pin for output.
  pinMode(TRIG_PIN, OUTPUT);
  digitalWrite(TRIG_PIN, LOW);
  
  // Initialize the echo pin for input.
  pinMode(ECHO_PIN, INPUT);
}

// ================================================================================
/// Update function to poll from loop().
void poll_sonar(unsigned long interval)
{
  // Test whether to run a measurement cycle.
  sonar_timer -= interval;
  
  if (sonar_timer <= 0) {

    // Reset the timer for the next sampling period.
    sonar_timer += sonar_interval;

    // Generate a short trigger pulse.
    digitalWrite(TRIG_PIN, HIGH);
    delayMicroseconds(10);
    digitalWrite(TRIG_PIN, LOW);

    // Measure the pulse length in microseconds.
    long echo_time = pulseIn(ECHO_PIN, HIGH, TIMEOUT);

    // If valid, scale into real-world units.
    if (echo_time > 0) {

      // Convert to a distance.  Note that the speed of sound is specified in
      // cm/sec, so the duration is scaled from microsecondst o seconds.  The
      // factor of 2 accounts for the round-trip doubling the time.
      sonar_distance = (echo_time * 1e-6 * SOUND_SPEED) / 2;
      Serial.print("sonar ");
      Serial.println(sonar_distance);

    } else {
      sonar_distance = -1;
      Serial.println("sonar none");
    }
  }
}
// ================================================================================
